using System;
using System.Drawing;
using System.Windows.Forms;
using C1.Win.C1FlexGrid;

namespace FlexFreezeBottomSample
{
	/// <summary>
	/// 
	/// FlexFreezeRight
	/// 
	/// This is a control (derived from Panel) that contains a FlexGrid. The
	/// contained FlexGrid is positioned with a negative X coordinate within 
	/// the panel so that the fixed columns are not visible.
	/// 
	/// The contained grid is also sized and scrolled via code so that the 
	/// right 'n' columns are visible.
	/// 
	/// The contained grid is bound to a main FlexGrid and therefore shares 
	/// the underlying grid model (data, layout, styles, selection, etc).
	/// 
	/// The whole control (panel and contained grid) is attached to the main 
	/// FlexGrid and positioned at its right. This way, the control always
	/// shows the right 'n' columns of the main grid.
	/// 
	/// </summary>
	internal class FlexFreezeRight : Panel
	{
		private C1.Win.C1FlexGrid.C1FlexGrid _frz;
		private C1.Win.C1FlexGrid.C1FlexGrid _main;
		private int  _cols;
		internal FlexFreezeRight(C1.Win.C1FlexGrid.C1FlexGrid main, int cols)
		{
			// save info we'll need later
			_main = main;
			_cols = cols;

			// create child grid with frozen bottom rows
			_frz = new C1.Win.C1FlexGrid.C1FlexGrid();
			_frz.ScrollBars = ScrollBars.None;
			_frz.BorderStyle = C1.Win.C1FlexGrid.Util.BaseControls.BorderStyleEnum.None;
			_frz.DrawMode = DrawModeEnum.OwnerDraw;
			_frz.OwnerDrawCell += new C1.Win.C1FlexGrid.OwnerDrawCellEventHandler(_frz_OwnerDrawCell);
			_frz.SetupEditor   += new C1.Win.C1FlexGrid.RowColEventHandler(_frz_SetupEditor);
			_frz.AfterScroll   += new C1.Win.C1FlexGrid.RangeEventHandler(_frz_AfterScroll);
			_frz.GridChanged   += new C1.Win.C1FlexGrid.GridChangedEventHandler(_main_GridChanged);
			Controls.Add(_frz);

			// bind frozen grid to main grid (share data, layout, styles, selection)
			_frz.DataSource = main;

			// add this to the main grid
			main.Controls.Add(this);

			// hook up the events we need to synchronize with the main grid
			main.Resize      += new System.EventHandler(_main_Resize);
			main.VisibleChanged += new System.EventHandler(_main_Resize);
			main.AfterScroll += new C1.Win.C1FlexGrid.RangeEventHandler(_main_AfterScroll);
			main.SelChange   += new System.EventHandler(_main_SelChange);
			main.GridChanged += new C1.Win.C1FlexGrid.GridChangedEventHandler(_main_GridChanged);

			// initialize
			AdjustLayout();
		}

		internal void AdjustLayout()
		{
			// get index of the right (frozen) columns
			ColumnCollection cols = _main.Cols;
			int c2 = cols.Count-1;
			int c1 = c2 - _cols + 1;
			if (c1 <= cols.Fixed) c1 = cols.Fixed;

			// hide control if the main grid is big enough
			Visible = (c1 <= c2);
			if (!Visible) return;

			// set control size and location
			Rectangle rc = _main.ClientRectangle;
			if (cols[c2].Right <= rc.Right)
			{
				Visible = false;
				return;
			}
			int wid = cols[c2].Right - cols[c1].Left + 1;
			rc.X = rc.Right - wid;
			rc.Width = wid;
			Bounds = rc;

			// adjust child grid to hide fixed columns
			rc = ClientRectangle;
			wid = cols[cols.Fixed + cols.Frozen].Right;
			rc.X -= wid;
			rc.Width += wid;
			_frz.Bounds = rc;

			// syncronize scroll position
			SynchScrollPosition(false);
		}
		internal void SynchScrollPosition(bool synchMain)
		{
			Point pos;

			// optionally synchronize Y scroll position on the main grid
			if (synchMain)
			{
				pos = _frz.ScrollPosition;
				pos.X = _main.ScrollPosition.X;
				_main.ScrollPosition = pos;
			}

			// syncronize Y scroll position of the frozen grid with main
			pos = _main.ScrollPosition;
			pos.X = -64000;
			_frz.ScrollPosition = pos;

			// update everything right away
			_main.Update();
			_frz.Update();
		}

		// make normal and scrollable areas look different
		// (when painting and when editing)
		private void _frz_OwnerDrawCell(object sender, C1.Win.C1FlexGrid.OwnerDrawCellEventArgs e)
		{
			if (e.Style.BackColor == _frz.Styles.Normal.BackColor)
			{
				e.Graphics.FillRectangle(Brushes.Beige, e.Bounds);
				e.DrawCell(DrawCellFlags.Border | DrawCellFlags.Content);
			}
		}
		private void _frz_SetupEditor(object sender, C1.Win.C1FlexGrid.RowColEventArgs e)
		{
			_frz.BackColor = Color.Beige;
		}

		// adjust layout after resizing the main grid (our container)
		private void _main_Resize(object sender, System.EventArgs e)
		{
			AdjustLayout();
		}

		// synchronize scrolling
		private void _main_AfterScroll(object sender, C1.Win.C1FlexGrid.RangeEventArgs e)
		{
			SynchScrollPosition(false);
		}
		private void _frz_AfterScroll(object sender, C1.Win.C1FlexGrid.RangeEventArgs e)
		{
			// if the frozen grid has the focus and the user selects non-frozen
			// cells, scroll main grid up.
			if (_frz.Focused)
			{
				int col = Math.Min(_frz.Col, _frz.ColSel);
				int rig = _main.Cols[col].Right + _main.ScrollPosition.X;
				if (rig > Top)
				{
					// unless this is a mouse selection, give focus to main grid
					if (!_frz.Capture) _main.Focus();
					_main.LeftCol = _main.Col;
				}
			}
			SynchScrollPosition(true);
		}

		// scroll while selecting
		// if the user extends the selection down on the main grid, scroll it down.
		private void _main_SelChange(object sender, System.EventArgs e)
		{
			// only when the main grid has the focus!
			if (!_main.Focused) return;

			// calculate selection bottom (accounting for scrolling)
			int col = Math.Max(_main.Col, _main.ColSel);
			int rig = _main.Cols[col].Right + _main.ScrollPosition.X;

			// try scrolling if past the bottom of the frozen area
			if (rig > Left) _main.LeftCol++;
		}

		// adjust layout after massive changes (resize rows/cols, sort, etc)
		private void _main_GridChanged(object sender, C1.Win.C1FlexGrid.GridChangedEventArgs e)
		{
			switch (e.GridChangedType)
			{
				case GridChangedTypeEnum.GridChanged:
				case GridChangedTypeEnum.LayoutChanged:
					AdjustLayout();
					break;
			}
		}
	}
}